module BotMob
  module Networks
    module Slack
      # # BotMob::Networks::Slack::Connection
      #
      # Handles all the connection details between a bot and Slack
      class Connection < BotMob::Connection
        attr_reader :bot, :token

        def initialize(bot, options = {})
          @bot = bot
          @token = options[:token]
          raise BotMob::NetworkConnectionError, 'Slack connection requires a :token parameter' if @token.nil?
        end

        def deliver(outbound_message, options = {})
          BotMob.logger.info("  Sending to Slack: \"#{options.inspect}\"\n")
          BotMob.logger.info("  Text: \"#{outbound_message.body.inspect}\"\n")

          client.chat_postMessage(
            channel: outbound_message.channel,
            text: outbound_message.body,
            as_user: true
          )
        end

        def client
          @client ||= begin
            client = ::Slack::Web::Client.new(token: token)
            client.auth_test
            client
          end
        end
      end
    end
  end
end
