module BotMob
  # ## BotMob::Roster
  #
  # This manages the pool of bots made available
  # by the application
  class Roster
    attr_reader :bots

    def initialize
      @bots = []
    end

    def register(bot, **options)
      @bots << (bot.is_a?(Class) ? bot.new(options) : bot)
    end

    # ## `disseminate`
    #
    # This method will distribute a message or command to all bots
    # on the roster. It collects and returns a compacted array
    # of the bots' responses
    def disseminate(action, params = {})
      bots.map do |bot|
        approach = "handle_#{action}".to_sym
        respond_to?(approach, true) ? send(approach, bot, params) : nil
      end.compact
    end

    private

    def handle_message(bot, params)
      bot.receive(params)
    end

    def handle_command(bot, params)
      method_name = params[:method].to_sym
      bot.receive_command(method_name, params)
    end
  end
end
